#include "avsFineTuningTool.hpp"
#include "avsUpdateTuning.hpp"
#include <iostream>
#include <fstream>
#include <string>
#include <algorithm>
#include <json.hpp>

void readJsonFineTuningRecord(std::string path, nlohmann::json& record) {
    using json = nlohmann::json;
    std::ifstream i(path);
    i >> record;

    std::cout << "Before update Fine Show..." << std::endl;

    for (const auto& item : record["FineTuning"]) {
        int index = item["INDEX"];
        int yaw = item["YAW"];
        int pitch = item["PITCH"];
        int roll = item["ROLL"];
        int offsetX = item["OFFSETX"];
        int offsetY = item["OFFSETY"];
        int distance = item["DISTANCE"];
        std::cout << "*********************CUT OFF LINE*********************" << std::endl;
        std::cout << "INDEX: " << index << std::endl;
        std::cout << "YAW: " << yaw << std::endl;
        std::cout << "PITCH: " << pitch << std::endl;
        std::cout << "ROLL: " << roll << std::endl;
        std::cout << "OFFSETX: " << offsetX << std::endl;
        std::cout << "OFFSETY: " << offsetY << std::endl;
        std::cout << "DISTANCE: " << distance << std::endl;
    }
}

void writeJsonFileToRecordFineTuning(nlohmann::json& record, int index, int yaw, int pitch, int roll, int offsetX, int offsetY, int distance, std::string path) {
    using json = nlohmann::json;
    std::ifstream i(path);
    json j;
    i >> j;

    for (auto& item : j["FineTuning"]) {
        if (item["INDEX"] == index) {
            item["YAW"] = yaw;
            item["PITCH"] = pitch;
            item["ROLL"] = roll;
            item["OFFSETX"] = offsetX;
            item["OFFSETY"] = offsetY;
            item["DISTANCE"] = distance;
            break;
        }
    }

    std::ofstream o(path);
    o << j.dump(4);
}




int main(int argc, char* argv[]) {

    std::cout << "Usage: " << argv[0] << " -INDEX <value> -YAW <value> -PITCH <value> -ROLL <value> -OFFSETX <value> -OFFSETY <value> -DISTANCE <value> -PATH <value>" << std::endl;

    if (argc < 2) {
        std::cerr << "No parameters provided" << std::endl;
        return 1;
    }

    double index = 0, yaw = 0, pitch = 0, roll = 0, offsetX = 0, offsetY = 0, distance = 0;
    std::string path;
    for (int i = 1; i < argc; i += 2) {
        std::string arg = argv[i];
        std::transform(arg.begin(), arg.end(), arg.begin(), ::toupper);
        if (arg == "-INDEX") {
            index = std::stoi(argv[i + 1]);
        } else if (arg == "-YAW") {
            yaw = std::stod(argv[i + 1]);
        } else if (arg == "-PITCH") {
            pitch = std::stod(argv[i + 1]);
        } else if (arg == "-ROLL") {
            roll = std::stod(argv[i + 1]);
        } else if (arg == "-OFFSETX") {
            offsetX = std::stod(argv[i + 1]);
        } else if (arg == "-OFFSETY") {
            offsetY = std::stod(argv[i + 1]);
        } else if (arg == "-DISTANCE") {
            distance = std::stod(argv[i + 1]);
            if(distance <= 0.5 || distance >= 50000){
                std::cerr << "Distance value must be between 0.5 and 10000" << std::endl;
                return -1;
            }
        } else if(arg == "-PATH"){
            path = argv[i + 1];
            std::cout << "SAVE AVSP PATH: " << path << std::endl;
        }else {
            std::cerr << "Unknown parameter: " << arg << std::endl;
            return -1;
        }
    }

    nlohmann::json record;
    // Further processing with the parsed parameters
    readJsonFineTuningRecord("./fineTuningRecord.json",record);
    writeJsonFileToRecordFineTuning(record, index, yaw, pitch, roll, offsetX, offsetY, distance, "./fineTuningRecord.json");
    std::cout << "*********************CUT OFF LINE*********************" << std::endl;

    std::cout << "Fine Tuning Record Updated..." << std::endl;

    std::cout << "INDEX: " << index << std::endl;
    std::cout << "YAW: " << yaw << std::endl;
    std::cout << "PITCH: " << pitch << std::endl;
    std::cout << "ROLL: " << roll << std::endl;
    std::cout << "OFFSETX: " << offsetX << std::endl;
    std::cout << "OFFSETY: " << offsetY << std::endl;
    std::cout << "DISTANCE: " << distance << std::endl;

    AvsFineTuningTool* avsFineTuningTool = AvsFineTuningTool::getInstance();

    avsFineTuningTool->setStitchingDistance(distance);
    avsFineTuningTool->SetYaw(index, yaw);
    avsFineTuningTool->SetPitch(index, pitch);
    avsFineTuningTool->SetRoll(index, roll);
    avsFineTuningTool->SetOffsetX(index, offsetX);
    avsFineTuningTool->SetOffsetY(index, offsetY);
    avsFineTuningTool->setAVSPLUTSavePath(path);

    avsFineTuningTool->GenerateLutFile(index);

    AvsUpdateTuning avsUpdateTuning(0);

    avsUpdateTuning.setPath(path);

    avsUpdateTuning.updateGrpAttr();

    return 0;

}